/*
 *
 *  KBluetooth4 - KDE Bluetooth Framework
 *
 *  Copyright (C) 2008  Tom Patzig <tpatzig@suse.de>
 *
 *  This file is part of kbluetooth4.
 *
 *  kbluetooth4 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  kbluetooth4 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with kbluetooth4; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
*/


#include "trayicon.h"
#include <kstatusbarjobtracker.h>
#include <KIO/JobUiDelegate>
#include <KNotification>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Tag>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/NAO>
#include <KMenu>
#include <KIconLoader>
#include <KIconEffect>
#include <KTemporaryFile>

#include "obexserver/obexserversessionfiletransfer.h"

KBlueTray::KBlueTray(const QString& path, QObject* parent) : KNotificationItem(parent), agentPath(path)
{
	adapter = 0;
	server = 0;
	serversession = 0;
        agent = 0;
	adapterConfig = 0;
	session = 0;
	progress = 0;
	devSelector = 0;
	lockSelector = 0;
	m_kBlueLock = 0;
	
	modes << "Off" << "Discoverable" << "Connectable" ;
	
	setParent(kapp);
	setIconByName("kbluetooth4");
	setToolTip("kbluetooth4", "KBluetooth", "KDE bluetooth framework");
	setCategory(KNotificationItem::Hardware);
	 
	QDir dir(QDir::homePath());
	dir.mkdir(".kbluetooth4");

        QDBusConnection::systemBus().registerObject(agentPath, kapp);

	initIcons();
	initConfig();
	initMenu();

	Solid::Control::BluetoothManager &man = Solid::Control::BluetoothManager::self();
	dbus = new QDBusConnection("dbus");

	connect(&man,SIGNAL(interfaceAdded(const QString&)),this,SLOT(adapterAdded(const QString&)));
	connect(&man,SIGNAL(interfaceRemoved(const QString&)),this,SLOT(adapterRemoved(const QString&)));
	connect(&man,SIGNAL(defaultInterfaceChanged(const QString&)),this,SLOT(defaultAdapterChanged(const QString&)));

	kblueLockEnabled = false;
	
	if ( man.bluetoothInterfaces().size() > 0 ) {
                defaultAdapterUBI = man.defaultInterface();
		onlineMode();
		if ( config->group("KBlueLock").readEntry("LockEnabled",false) && ! config->group("KBlueLock").readEntry("Device","").isEmpty() ) {
			lockEnableAction->setChecked(true);
			m_kBlueLock = new KBlueLock(adapter);
			lockDevice = m_kBlueLock->lockDevice();
			connect(m_kBlueLock, SIGNAL(lockDisabled()), this, SLOT(lockDisabled()));
			connect(m_kBlueLock, SIGNAL(lockReady()), this, SLOT(lockReady()));
			m_kBlueLock->enable();
			lockEnabled();
		}
	} else {
		offlineMode();
	}
}

KBlueTray::~KBlueTray()
{

	if (adapter) {
                adapter->unregisterAgent(agentPath);
		adapter->stopDiscovery();
		delete adapter;
	}
        if (agent)
            delete agent;

	if (session)
		delete session;
	if (progress)
		delete progress;
	if (devSelector)
//		delete devSelector;
	if (lockSelector)
//		delete lockSelector;

	if(server) {
		server->stop();
		server->close();
		delete server;
	}
	
	if(serversession) {
		serversession->cancel();
		serversession->disconnect();
		delete serversession;
	}
	
	delete confGroup;
	delete config;
	
	if(m_kBlueLock)
		delete m_kBlueLock;
	
	qDebug() << "bye bye";
}

void KBlueTray::initIcons() {
	KIconLoader loader;
	QString iconPath = KIconLoader::global()->iconPath( "kbluetooth4", KIconLoader::Panel );
	m_IconEnabled = QIcon( iconPath );
	QImage iconDisabled = m_IconEnabled.pixmap( loader.currentSize( KIconLoader::Panel ) ).toImage();
	KIconEffect::toGray( iconDisabled, 1.0 );
	m_IconDisabled = QIcon( QPixmap::fromImage( iconDisabled ) );
}

void KBlueTray::initMenu() {
	kbluelockMenu = contextMenu()->addMenu(KIcon("preferences-desktop-user-password"), i18n("KBlueLock"));

	lockEnableAction = kbluelockMenu->addAction(KIcon("system-lock-screen"), i18n("Enable"), this, SLOT(enableLock()));
	lockEnableAction->setCheckable(true);
	lockEnableAction->setChecked(false);

	lockConfigureAction = kbluelockMenu->addAction(KIcon("configure"), i18n("Configure..."), this, SLOT(slotConfigureKBlueLock()));
	lockConfigureAction->setDisabled(true);

	m_sendMenu = contextMenu()->addMenu(KIcon("text-directory"), i18n("Send"));
	
	sendToAction = m_sendMenu->addAction(KIcon("text-directory"), i18n("File"), this, SLOT(sendFile()));
	sendToAction->setDisabled(false);
	
	m_sendClipboardText = m_sendMenu->addAction(KIcon("edit-paste"), i18n("Clipboard Text"), this, SLOT(sendClipboardText()));

	wizardAction = contextMenu()->addAction(KIcon("kbluetooth4"), i18n("Device Manager"), this, SLOT(showManager()));
	wizardAction->setDisabled(false);

	settingsMenu = contextMenu()->addMenu(KIcon("preferences-system"), i18n("Settings"));

	adapterAction = settingsMenu->addAction(KIcon("configure"), i18n("Bluetooth Adapters"), this, SLOT(showAdapterInterface()));

	serverAction = settingsMenu->addAction(KIcon("network-server"), i18n("Obex server"), this, SLOT(enableServer()));
	serverAction->setCheckable(true);
	KConfigGroup obexServerConfig(config, "ObexServer");
	if(obexServerConfig.readEntry("Autostart") == "true") {
		serverAction->setChecked(true);
	}

	aboutAction = contextMenu()->addAction(KIcon("folder-remote"), i18n("About"), this, SLOT(showAboutInfo()));

	/*quitAction = contextMenu()->addAction(KIcon("application-exit"),i18n("Quit"));
	connect(quitAction, SIGNAL(triggered(bool)), this, SLOT(slotQuitApp(bool)));*/
}

void KBlueTray::initConfig() {
	config = new KConfig("kbluetooth4rc");
	confGroup = new KConfigGroup(config, "General");
	KConfigGroup obexServerConfig(config, "ObexServer");
	if(!obexServerConfig.hasKey("savePath")) {
		obexServerConfig.writeEntry("savePath", QDir::homePath() + "/.kbluetooth4");
		obexServerConfig.sync();
	}
	if(!obexServerConfig.hasKey("Autostart")) {
		obexServerConfig.writeEntry("Autostart", "true");
		obexServerConfig.sync();
	}
	if(!config->group("KBlueLock").hasKey("Device")) {
		config->group("KBlueLock").writeEntry("Device", "");
		config->group("KBlueLock").sync();
	}
	if(!config->group("KBlueLock").hasKey("LockEnabled")) {
		config->group("KBlueLock").writeEntry("LockEnabled", false);
		config->group("KBlueLock").sync();
	}
	if(!config->group("KBlueLock").hasKey("UnlockEnabled")) {
		config->group("KBlueLock").writeEntry("UnlockEnabled", true);
		config->group("KBlueLock").sync();
	}
}

void KBlueTray::onlineMode()
{
	kDebug() << "online Mode";
	Solid::Control::BluetoothManager &man = Solid::Control::BluetoothManager::self();	
	kDebug() << "adapter size " << Solid::Control::BluetoothManager::self().bluetoothInterfaces().size();

	online = true;
	setStatus(KNotificationItem::Active);
	setIconByPixmap( m_IconEnabled );
	
	adapter = new Solid::Control::BluetoothInterface(man.defaultInterface());
	connect(adapter,SIGNAL(propertyChanged(const QString&, const QVariant&)),this,SLOT(slotPropertyChanged(const QString&, const QVariant&)));

	kDebug() << "Adapter found " << adapter->name();

        adapter->registerAgent(agentPath,"DisplayYesNo");
        if (!agent)
            agent = new Agent(kapp,adapter);
        agent->setBluetoothInterface(adapter);
        agent->setExitOnRelease(false);
	
	updateTooltip();
	
	sendToAction->setEnabled(true);
	wizardAction->setEnabled(true);
	adapterAction->setEnabled(true);
	kbluelockMenu->setEnabled(true);
        settingsMenu->setEnabled(true);

        if (!server) {
            server = new ObexServer(this, adapter->address(), "opp", false);
            connect(server, SIGNAL(serverReady()), this, SLOT(slotServerReady()));
            connect(server, SIGNAL(started()), this, SLOT(slotServerStarted()));
            connect(server, SIGNAL(stopped()), this, SLOT(slotServerStopped()));
            connect(server, SIGNAL(closed()), this, SLOT(slotServerClosed()));
            connect(server, SIGNAL(sessionCreated(const QString&)), this, SLOT(slotServerSessionCreated(const QString&)));
            connect(server, SIGNAL(sessionRemoved(const QString&)), this, SLOT(slotServerSessionRemoved(const QString&)));
            connect(server, SIGNAL(errorOccured(const QString&, const QString&)), this, SLOT(slotServerErrorOccured(const QString&, const QString&)));
        } else {
		KConfigGroup obexServerConfig(config, "ObexServer");
		server->start(config->group("ObexServer").readEntry("savePath"), true, false);
	}



}

void KBlueTray::offlineMode()
{
	kDebug() << "offline Mode";

	online = false;

	sendToAction->setEnabled(false);
	wizardAction->setEnabled(false);
	adapterAction->setEnabled(false);
	kbluelockMenu->setEnabled(false);
	settingsMenu->setEnabled(false);
	
	setStatus(KNotificationItem::Passive);
	setIconByPixmap( m_IconDisabled );
		
	if (adapter) {
                kDebug() << "Unregistering Agent";
                adapter->unregisterAgent(agentPath);
		disconnect(adapter,0,0,0);
		delete adapter;
		adapter = 0;
	}
	
	if(server) {
		server->stop();
	}

	if(serversession) {
		serversession->cancel();
		serversession->disconnect();
		delete serversession;
		serversession = 0;
	}

}

void KBlueTray::updateTooltip() {
        kDebug() << "Updating Tooltip";	
	if(!online) {
		toolTipInfo = i18n("No Bluetooth Adapter");
		return;
	}

        if(!adapter || !server ) 
            return;
	
	
	toolTipInfo = 	"<b>Name:</b> " + adapter->name() + "<br />" \
			+	"<b>Address:</b> " + adapter->address() + "<br />" \
			+	"<b>KBlueLock:</b> ";

	if (!kblueLockEnabled)
		toolTipInfo += "Disabled"; 
	else
		toolTipInfo += ("Enabled for " + lockDeviceName);
		
	toolTipInfo += "<br /><b>Receiving Files:</b> ";
	if(server) {
		toolTipInfo += ((server->isStarted()) ? "Enabled" : "Disabled");
	} else {
		toolTipInfo += "Disabled";
	}
	
	//tray->setToolTip(toolTipInfo);
}

void KBlueTray::adapterAdded(const QString& ubi)
{
	 kDebug() << "adapterAdded: " << ubi;

}

void KBlueTray::adapterRemoved(const QString& ubi)
{
	 kDebug() << "adapterRemoved: " << ubi;
	 Solid::Control::BluetoothManager &man = Solid::Control::BluetoothManager::self();
	 
	 if (man.bluetoothInterfaces().size() == 0) {
		offlineMode();
		kDebug() << "default Adapter removed";
	 }
	updateTooltip();
}

void KBlueTray::defaultAdapterChanged(const QString& ubi)
{
	kDebug() << "default Adapter changed " << ubi;
        if (!online) {
	    kDebug() << "new default adapter UBI: " << ubi;
    	    defaultAdapterUBI = ubi;
            onlineMode();
        }
}

void KBlueTray::slotQuitApp(bool /*set*/)
{
        kapp->quit();
}

void KBlueTray::showAboutInfo()
{
        /*KAboutData aboutData("kbluetooth4", 0, ki18n("KBluetooth4"), "0.3",
                           ki18n("KDE4 Bluetooth Framework"), KAboutData::License_GPL_V2);
        aboutData.addAuthor(ki18n("Tom Patzig"), KLocalizedString(), "tpatzig@suse.de", "http://www.kde.org/");
	aboutData.addCredit(ki18n("Arjan Topolovec"), KLocalizedString(), "arjan.top@gmail.com", "");
        aboutData.setProgramLogo(KIcon("kbluetooth4"));

        KAboutApplicationDialog about(&aboutData, mainmenu);
//        about.setIcon( KIcon( "kbluetooth4" ) );
        about.exec();*/
}

void KBlueTray::showAdapterInterface()
{
	if (!adapterConfig) {
		adapterConfig = new AdapterConfig(this);
                connect(adapterConfig,SIGNAL(adapterConfigClosed()),this,SLOT(updateTooltip()));
        }

	adapterConfig->initialize();
}

void KBlueTray::showWizard()
{
//bluez4//	BTWizard wiz;
//bluez4//	wiz.exec();
}

void KBlueTray::showManager()
{
//	DeviceMan manager;
	kDebug() << "Starting Device Manager";
	KProcess process;
	process.setProgram("kbluetooth4-devicemanager");
	process.startDetached();
	
//	manager.exec();
}

//  ################# SEND FILE ################################

void KBlueTray::sendFile()
{
	QString fileName = KFileDialog::getOpenFileName(KUrl("./"), "*", 0, tr("Select File"));
        if (!fileName.isEmpty()) {
		fileToSend = fileName;
		devSelector = new DeviceSel(this,QString("computer,phone").split(','));
	 	devSelector->hideExtraCheckBox();
		devSelector->setInfoLabel(i18n("Selecting a device starts the file transfer."));
		connect(devSelector,SIGNAL(deviceSelected(const QString&)),this,SLOT(slotSendFile(const QString&)));
        }


}

void KBlueTray::slotSendFile(const QString& mac)
{
	session = new ObexSession(this,adapter->address(),mac,"opp");
	connect(session,SIGNAL(connected()),this,SLOT(obexSessionReady()));
}

void KBlueTray::sendClipboardText()
{
	devSelector = new DeviceSel(this, QString("computer,phone").split(','));
	devSelector->hideExtraCheckBox();
	devSelector->setInfoLabel(i18n("Selecting a device starts the file transfer."));
	connect(devSelector, SIGNAL(deviceSelected(QString)), this, SLOT(slotSendClipboardText(QString)));
}

void KBlueTray::slotSendClipboardText(const QString& address)
{
	KTemporaryFile tempFile;
	tempFile.setAutoRemove(false);
	QTextStream stream(&tempFile);
	if(tempFile.open()) {
		stream << QApplication::clipboard()->text();
		kDebug() << "Clipboard file: " << tempFile.fileName();
		fileToSend = tempFile.fileName();
		tempFile.close();
	}
	slotSendFile(address);
}

void KBlueTray::obexSessionReady()
{
	//kDebug() << "is connected: " << session->isConnected();
	//if(!session->isConnected()) return;

	KJob* fileTransfer = session->sendFile(fileToSend);
	connect(fileTransfer, SIGNAL( result(KJob*) ), this, SLOT( slotFileTransferCompleted(KJob*) ));
	KIO::getJobTracker()->registerJob(fileTransfer);
	fileTransfer->start();
}

void KBlueTray::slotFileTransferCompleted(KJob* ) {
	delete session;
	session = 0;
	delete devSelector;
	devSelector = 0;
}


// ######################################### KBlueLock #####################################################

void KBlueTray::enableLock()
{
	if (lockEnableAction->isChecked() ) {
		m_kBlueLock = new KBlueLock(adapter);
		lockDevice = m_kBlueLock->lockDevice();
		if (lockDevice.isEmpty()) {
			kDebug() << "Configuring new KBlueLock Device";
			lockSelector = new DeviceSel(this,QString("computer,phone").split(','));
			lockSelector->setOkButtonText(i18n("Select"));
			lockSelector->setInfoLabel(i18n("Selecting a device enables KBlueLock (a balloon message will pop up from the tray when your target device is found and KBlueLock is ready).\nIf the device disappears, your screen will be locked."));

			connect(lockSelector, SIGNAL(deviceSelected(QString)), m_kBlueLock, SLOT(enable(QString)));
			connect(lockSelector,SIGNAL(selectorCancelled()),this,SLOT(slotBlueSelectorCancelled()));
			connect(lockSelector, SIGNAL(unlockChanged(bool)), m_kBlueLock, SLOT(unlockEnable(bool)));
			
			connect(m_kBlueLock, SIGNAL(lockEnabled()), this, SLOT(lockEnabled()));
			connect(m_kBlueLock, SIGNAL(lockDisabled()), this, SLOT(lockDisabled()));
			connect(m_kBlueLock, SIGNAL(lockReady()), this, SLOT(lockReady()));
		} else {
			connect(m_kBlueLock, SIGNAL(lockDisabled()), this, SLOT(lockDisabled()));
			connect(m_kBlueLock, SIGNAL(lockReady()), this, SLOT(lockReady()));
			m_kBlueLock->enable();
			lockEnabled();
		}
	} else {
		m_kBlueLock->disable();
	}
	updateTooltip();
}

void KBlueTray::lockEnabled()
{
	QString iconPath = KIconLoader::global()->iconPath( "document-encrypt", KIconLoader::Panel );
	setOverlayIconByPixmap( QIcon(iconPath).pixmap(14, 14) );
	setIconByPixmap(m_IconEnabled);
	showMessage("KBluetooth", "kbluelock is looking for the target device...", "kbluetooth4");
	lockConfigureAction->setEnabled(true);
}

void KBlueTray::lockDisabled()
{
  	lockConfigureAction->setEnabled(false);
	kDebug() << "Lock disabled";
	setOverlayIconByPixmap(QIcon());
	setIconByPixmap(m_IconEnabled);
	delete m_kBlueLock;
	m_kBlueLock = 0;
}

void KBlueTray::lockReady()
{
	showMessage("KBluetooth", "kbluelock is enabled now", "kbluetooth4");
}

void KBlueTray::slotConfigureKBlueLock()
{
                m_kBlueLock->disable();
                kDebug() << "Periodic Discovery Stopped";
		
		m_kBlueLock = new KBlueLock(adapter);
		m_kBlueLock->unlockEnable(true);

		lockSelector = new DeviceSel(this,QString("computer,phone").split(','));
		lockSelector->setOkButtonText("Select");
		lockSelector->setInfoLabel(i18n("Selecting a device enables KBlueLock (a balloon message will pop up from the tray when your target device is found and KBlueLock is ready).\nIf the device disappears, your screen will be locked."));
		
		connect(lockSelector, SIGNAL(deviceSelected(QString)), m_kBlueLock, SLOT(enable(QString)));
		connect(lockSelector,SIGNAL(selectorCancelled()),this,SLOT(slotBlueSelectorCancelled()));
		connect(lockSelector, SIGNAL(unlockChanged(bool)), m_kBlueLock, SLOT(unlockEnable(bool)));
		
		connect(m_kBlueLock, SIGNAL(lockEnabled()), this, SLOT(lockEnabled()));
		connect(m_kBlueLock, SIGNAL(lockDisabled()), this, SLOT(lockDisabled()));
		connect(m_kBlueLock, SIGNAL(lockReady()), this, SLOT(lockReady()));
}

void KBlueTray::slotBlueSelectorCancelled()
{

	disconnect(lockSelector, SIGNAL(deviceSelected(QString)), m_kBlueLock, SLOT(enable(QString)));
	disconnect(lockSelector,SIGNAL(selectorCancelled()),this,SLOT(slotBlueSelectorCancelled()));
	disconnect(lockSelector, SIGNAL(unlockChanged(bool)), m_kBlueLock, SLOT(unlockEnable(bool)));
	disconnect(lockSelector,0,0,0);
	kblueLockEnabled = false;
	//enableBlueLock(lockDevice);

}

//################# external SIGNALS ####################################

void KBlueTray::slotPropertyChanged(const QString& prop, const QVariant& value)
{
    kDebug() << "Property Changed: " << prop << value;
    if (prop == "Name")
        updateTooltip();
}


//################### RECEIVE FILES #############################################

void KBlueTray::slotServerReady() {
	if(config->group("ObexServer").readEntry("Autostart") == "true") {
		if(!server->isStarted()) {
			server->start(config->group("ObexServer").readEntry("savePath"), true, false);
		}
	}
	updateTooltip();
}

void KBlueTray::slotServerErrorOccured(const QString& error_name, const QString& error_message) {
	kDebug() << error_name << ": " << error_message;
	KMessageBox::error(0, error_name, error_message);
}

void KBlueTray::enableServer() {
	if(serverAction->isChecked()) {
		config->group("ObexServer").writeEntry("Autostart", "true");
		slotServerReady();
	} else {
		config->group("ObexServer").writeEntry("Autostart", "false");
		server->stop();
	}
}

void KBlueTray::slotServerStarted() {
	if(!server->isStarted()) {
	      kDebug() << "obex server not started";
	      server->close();
	} else {
	      kDebug() << "obex server started";
	      serverAction->setChecked(true);
	}
	updateTooltip();
}

void KBlueTray::slotServerStopped() {
	serverAction->setChecked(false);
	updateTooltip();
	kDebug() << "obex server stopped";
}

void KBlueTray::slotServerClosed() {
	kDebug() << "obex server closed";
	serverAction->setChecked(false);
	updateTooltip();
	delete server;
	server = 0;
}

void KBlueTray::slotServerSessionCreated(const QString& path) {
	kDebug() << path;
	serversession = new ObexServerSession(this, server, path);
	connect(serversession, SIGNAL(disconnected()), this, SLOT(slotServerTransferDisconnected()));
	connect(serversession, SIGNAL(transferStarted(KJob*)), this, SLOT(slotServerSessionTransferStarted(KJob*)));
	connect(serversession, SIGNAL(errorOccurred(const QString&, const QString&)), this, SLOT(slotServerErrorOccured(const QString&, const QString&)));
}

void KBlueTray::slotServerSessionRemoved(const QString& /*path*/) {
	if(serversession) {
		serversession->disconnect();
		delete serversession;
		serversession = 0;
	}
}

void KBlueTray::slotServerSessionTransferStarted(KJob* job) {
	ObexServerSessionFileTransfer* fileTransfer = static_cast<ObexServerSessionFileTransfer*>(job);
	int receiveFiles = KMessageBox::questionYesNo(0, "Receive files over bluetooth?", "Incoming files over Bluetooth");
        kDebug() << "Receive Files? " << receiveFiles;
	if(receiveFiles == 4) {
		QFile file(fileTransfer->localPath());
		if(!file.remove()) {
			kDebug() << "Error deleting file: " << fileTransfer->localPath();
		}
		fileTransfer->reject();
		serversession->cancel();
		serversession->disconnect();
		return;
	}
	kDebug() << "transfer started";
	connect(fileTransfer, SIGNAL( result(KJob*) ), this, SLOT( slotFileReceiveComplete(KJob*) ));
	KIO::getJobTracker()->registerJob(fileTransfer);
	fileTransfer->start();
	/*KNotification* notification = new KNotification("receiveFiles", 0, KNotification::Persistent);
	notification->setText("Recieve files over bluetooth?");
	notification->setPixmap( KIcon("kbluetooth4").pixmap(20, 20) );
	notification->setActions(QStringList() << "Yes" << "no");
	connect(notification, SIGNAL( activated(unsigned int) ), this, SLOT( slotAcceptFileRecieve(unsigned int) ));
	notification->sendEvent();*/
}

void KBlueTray::slotAcceptFileRecieve(unsigned int id) {
	if(id == 1) {
		//accept
	} else {
		//deny
	}
}

void KBlueTray::slotFileReceiveComplete(KJob* job) {
	ObexServerSessionFileTransfer* fileTransfer = static_cast<ObexServerSessionFileTransfer*>(job);
	if(Nepomuk::ResourceManager::instance()->init() == 0) {
		kDebug() << fileTransfer->localPath();
		Nepomuk::Resource file(QUrl(fileTransfer->localPath()));
		Nepomuk::Tag tag("bluetooth");
		tag.addSymbol("preferences-system-bluetooth");
		tag.setDescription("File received over bluetooth");
		file.addTag(tag);
	}
}

void KBlueTray::slotFileReceiveCancelled() {
	serversession->cancel();
	serversession->disconnect();
}

void KBlueTray::slotServerTransferDisconnected() {
	
}

